from hibeedb import models
from hibeetest import SimpleTestCase
from hibeetest.utils import isolate_apps


@isolate_apps("invalid_models_tests")
class CustomFieldTest(SimpleTestCase):
    def test_none_column(self):
        class NoColumnField(models.AutoField):
            def db_type(self, connection):
                # None indicates not to create a column in the database.
                return None

        class Model(models.Model):
            field = NoColumnField(primary_key=True, db_column="other_field")
            other_field = models.IntegerField()

        field = Model._meta.get_field("field")
        self.assertEqual(field.check(), [])
